import { useEffect, useRef, useState } from 'react';

const langChat = window.drupalSettings?.deepseek?.voice;
const urlAgent = window.drupalSettings?.deepseek?.url_agent || 'http://drupal.local/ajax/ai-connect';
const urlUpload = window.drupalSettings?.deepseek?.url_upload || 'http://drupal.local/deepseek/upload';

// Delay respone
const delay = (ms) => new Promise(resolve => setTimeout(resolve, ms));

// Create message
function createChatMessage(sessionId, content, role, file = {} ) {
  return {
    session: sessionId,
    datetime: new Date().toISOString(),
    content: content,
    role: role,
    file: file
  };
}

// Get format date
function getFormattedDate(offset = 0) {
  const date = new Date();
  date.setDate(date.getDate() - offset);
  return date.toISOString().split('T')[0];
}

// New session
function newSession(chat) {
  const today = getFormattedDate();
  const session = Date.now().toString(36) + Math.random().toString(36).slice(2, 7);
  chat.sessions[session] = {date: today, title: today};
  localStorage.setItem('chatAI', JSON.stringify(chat));
  return session;
}

// Get current session
function getCurrentSession(chat) {
  const today = getFormattedDate();
  if (Object.keys(chat.sessions).length) {
    let lastEntry = Object.entries(chat.sessions).at(-1);
    let lastSession = lastEntry?.[1];
    if (lastSession && lastSession.date === today) {
      return lastEntry?.[0];
    }
  }
  return newSession(chat);
}

// Strip HTML
function stripHtml(html) {
  const div = document.createElement('div');
  div.innerHTML = html;
  return div.textContent || div.innerText || '';
}

// Trim text
function trimText(text, maxLength = 40) {
  const plainText = stripHtml(text);
  if (plainText.length <= maxLength) return plainText;
  let trimmed = plainText.slice(0, maxLength);
  const lastSpaceIndex = trimmed.lastIndexOf(" ");
  if (lastSpaceIndex > 0) {
    trimmed = trimmed.slice(0, lastSpaceIndex);
  }
  return trimmed + "…";
}

export function useChatBot() {

  const chatLocal = localStorage.getItem('chatAI') ? JSON.parse(localStorage.getItem('chatAI')) : { chats: [], sessions: {} };

  const controllerRef = useRef(null);
  const recognitionRef = useRef(null);
  const blinkIntervalRef = useRef(null);
  const currentIconRef = useRef(null);
  const synthesisRef = useRef(null);

  const [chatAI, setChatAI] = useState({});
  const [messages, setMessages] = useState([]);
  const [thinkContent, setThinkContent] = useState('');
  const [session, setSessions] = useState(null);
  const [sessionHistory, setSessionsHistory] = useState(null);
  const [loading, setLoading] = useState('');
  const [loadingHistory, setLoadingHistory] = useState(false);
  const [voiceChat, setVoiceChat] = useState('');
  const [voiceStatus, setVoiceStatus] = useState(false);
  const [upFile, setUpFile] = useState([]);
  const [lang, setLang] = useState(localStorage.getItem('speechLanguage') || langChat || navigator.language);

  useEffect(() => {
    const getCurrent = getCurrentSession(chatLocal);
    setChatAI(chatLocal);
    setSessions(getCurrent);
    return () => {
      stopAll();
    };
  }, []);

  useEffect(() => {
    const data = session && chatAI.chats ? chatAI.chats.filter(chat => chat.session === session) : [];
    setMessages(data);
    setSessionsHistory(session);
  }, [session]);

  useEffect(() => {
    const SpeechRecognition = window.SpeechRecognition || window.webkitSpeechRecognition;

    if (!SpeechRecognition) {
      console.error("Browser doesn't support Speech Recognition");
      return;
    }

    const recognition = new SpeechRecognition();
    recognition.lang = lang;
    recognition.continuous = false;
    recognition.interimResults = false;

    recognition.onstart = () => {
      setVoiceStatus(true);
    };

    recognition.onspeechend = () => {
      if (recognitionRef.current) {
        recognitionRef.current.stop();
      }
      setVoiceStatus(false);
    };

    recognition.onend = () => {
      setVoiceStatus(false);
    };

    recognition.onresult = (event) => {
      const transcript = event.results[0][0].transcript;
      if (transcript) {
        setVoiceChat(transcript);
      }
    };

    recognitionRef.current = recognition;

    return () => {
      if (recognitionRef.current) {
        try {
          recognitionRef.current.abort();
        } catch (e) {
          console.error("Error during cleanup:", e);
        }
        recognitionRef.current = null;
      }
    };
  }, [lang]);

  // Call api
  const sendMessage = async (data) => {
    let fullResponse = '';
    const plainText = stripHtml(data);
    if(!plainText && !upFile.length) return;

    const newC = createChatMessage(session, plainText, 'user', upFile);
    setUpFile([]);
    setMessages(prev => [...prev, newC]);
    setLocalstorageChat(newC, plainText);

    const uniqueChats = [];
    const seen = new Set();
    chatAI.chats.forEach(chat => {
      if (chat.session === session && !seen.has(chat.content.trim().toLowerCase())) {
        seen.add(chat.content.trim().toLowerCase());
        uniqueChats.push(chat);
      }
    });

    const conversations = uniqueChats
      .map(chat => ({ content: chat.content, role: chat.role, file: chat.file}))
      .concat({ content: plainText, role: 'user', file: upFile});

    if (controllerRef.current) {
      controllerRef.current.abort();
    }

    const controller = new AbortController();
    controllerRef.current = controller;

    try {
      setLoading('loading');
      const response = await fetch(urlAgent, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'Accept': 'text/event-stream',
        },
        body: JSON.stringify(conversations),
        signal: controller.signal,
      });

      if (!response.ok) {
        setLoading('Network response was not ok');
        return;
      }

      const reader = response.body.getReader();
      const decoder = new TextDecoder('utf-8');
      let inThink = false;
      let linkContent = [];

      let updatedBotMessage = createChatMessage(session, '', 'assistant');
      setMessages(prev => [...prev, updatedBotMessage]);

      while (true) {
        const { done, value } = await reader.read();
        if (done) break;

        const chunk = decoder.decode(value, { stream: true });
        const lines = chunk.split('\n');
        for (const line of lines) {
          await delay(100);
          if (line.startsWith('data: ')) {
            try {
              const data = JSON.parse(line.substring(6));
              if (data.content === "<think>") {
                inThink = true;
                continue;
              }
              if (data.content === "</think>") {
                inThink = false;
                continue;
              }
              if (data.content === "[DONE]") {
                continue;
              }
              if (data.links) {
                linkContent = data.links;
              }
              if (typeof data.content === 'string') {
                if (inThink) {
                  setThinkContent(prev => prev + data.content);
                } else {
                  fullResponse += data.content;
                  updatedBotMessage = createChatMessage(session, fullResponse, 'assistant');
                  setMessages(prev => {
                    const updated = [...prev];
                    updated[updated.length - 1] = updatedBotMessage;
                    return updated;
                  });
                }
              }
            } catch (e) {
              console.error('Error parsing SSE data:', e);
            } finally {
              setLoading('');
            }
          }
        }
        if (linkContent.length) {
          updatedBotMessage.links = linkContent;
        }
      }
      setLocalstorageChat(updatedBotMessage);
    } catch (error) {
      setLoading('');
      console.error('Error fetching data:', error);
    } finally {
      controllerRef.current = null
    }
  };

  // New chat
  const newChat = () => {
    const data = newSession(chatAI);
    setSessions(data);
    stopAll();
  }

  // Remove chat
  const removeChat = () => {
    const chat = { chats: [], sessions: {} };
    setChatAI(chat);
    setSessions(newSession(chat));
    stopAll();
  }

  // Handle scroll
  const handleScroll = () => {
    if(!loadingHistory) {
      setLoadingHistory(true);
      const allSessions = [...new Set(chatAI.chats.map(chat => chat.session))];
      const currentIndex = allSessions.indexOf(sessionHistory);
      if (currentIndex > 0) {
        const sessionH = allSessions[currentIndex - 1];
        const messHistory = chatAI.chats.filter(chat => chat.session === sessionH)
        setSessionsHistory(sessionH);
        setMessages(prev => [...messHistory, ...prev]);
      }
      setTimeout(() => setLoadingHistory(false), 1000);
      return null;
    }
  }

  // Change languae
  const changeLang = (type) => {
    if (type && type !== lang) {
      setLang(type);
      localStorage.setItem('speechLanguage', type);
    }
  }

  // Save Localstorage
  function setLocalstorageChat(newChat, text = '') {
    setChatAI((prev) => {
      const updated = {
        ...prev,
        sessions: {
          ...prev.sessions,
          [session]: {
            ...prev.sessions[session],
            title: prev.sessions[session].date === prev.sessions[session].title ? trimText(text) : prev.sessions[session].title
          }
        },
        chats: [...prev.chats, newChat]
      };
      localStorage.setItem('chatAI', JSON.stringify(updated));
      return updated;
    });
  }

  // Up file
  const upLoad = async (file) => {
    const formData = new FormData();
    formData.append('file', file);
    try {
      const response = await fetch(urlUpload, {
        method: 'POST',
        body: formData,
        credentials: 'include',
      });
      const result = await response.json();
      if(result) {
        setUpFile(prev => {
          const exists = prev.some(file => file.path === result.path);
          if (exists) return prev;
          return [...prev, { name: result.name, path: result.path }];
        });
      }
    } catch (err) {
      console.error(err);
    }
  }

  // Start void
  const startVoice = () => {
    recognitionRef.current.start();
  }

  // Stop void
  const stopVoice = () => {
    recognitionRef.current.stop();
  }

  // Copy chat
  const copyChat = (event) => {
    const target = event.target;
    const messageBody = target.closest('.message-body');
    const messageContent = messageBody.querySelector('.message-content');
    const text = messageContent.innerText;
    if (!text) return;
    navigator.clipboard.writeText(text)
    .catch(err => {
      console.error('Failed to copy: ', err);
    });
    target.classList.replace('bi-clipboard','bi-clipboard-check');
    setTimeout(() => {
      target.classList.replace('bi-clipboard-check','bi-clipboard');
    }, 1000);
  }

  // Start edit
  const editChatStart = (event) => {
    const messageBody = event.target.closest('.message-body');
    if (!messageBody) return;
    const messageContent = messageBody.querySelector('.message-content');
    if (!messageContent) return;
    messageContent.setAttribute('contenteditable', true);
    messageContent.focus();
  }

  // Stop edit
  const editChatStop = (event, datetimeToUpdate) => {
    if (event.target.hasAttribute('contenteditable')) {
      event.target.setAttribute('contenteditable', false);
      const newContent = event.target.textContent.trim();
      chatAI.chats = chatAI.chats.map(chat => {
        if (chat.session === session && chat.datetime === datetimeToUpdate) {
          return { ...chat, content: newContent };
        }
        return chat;
      });
      localStorage.setItem("chatAI", JSON.stringify(chatAI));
    }
  }

  // Regenerate chat
  const regenerateChat = (event) => {
    const messageBody = event.target.closest('.message-body');
    if (!messageBody) return;
    const messageContent = messageBody.querySelector('.message-content');
    if (!messageContent) return;
    const text = messageContent.innerText;
    if (!text) return null;
    sendMessage(text);
  }

  // Speak chat
  const speakChat = (event) => {
    const messageBody = event.target.closest('.message-body');
    if (!messageBody) return;
    const messageContent = messageBody.querySelector('.message-content');
    if (!messageContent) return;
    const text = messageContent.innerText.trim();
    if (!text) return;

    if (speechSynthesis.speaking && synthesisRef.current?.text === text) {
      speechSynthesis.cancel();
      if (blinkIntervalRef.current) {
        clearInterval(blinkIntervalRef.current);
        blinkIntervalRef.current = null;
      }
      if (currentIconRef.current) {
        currentIconRef.current.classList.replace('bi-volume-up', 'bi-volume-down');
        currentIconRef.current = null;
      }
      return;
    }

    if (speechSynthesis.speaking) {
      speechSynthesis.cancel();
    }

    const utterance = new SpeechSynthesisUtterance(text);
    utterance.lang = lang;
    synthesisRef.current = utterance;

    utterance.onstart = () => {
      const icon = messageBody.querySelector('.btn-speak');
      currentIconRef.current = icon;

      blinkIntervalRef.current = setInterval(() => {
        if (icon.classList.contains('bi-volume-up')) {
          icon.classList.replace('bi-volume-up', 'bi-volume-down');
        } else {
          icon.classList.replace('bi-volume-down', 'bi-volume-up');
        }
      }, 500);
    };

    const resetBlink = () => {
      if (blinkIntervalRef.current) {
        clearInterval(blinkIntervalRef.current);
        blinkIntervalRef.current = null;
      }
      if (currentIconRef.current) {
        currentIconRef.current.classList.replace('bi-volume-up', 'bi-volume-down');
        currentIconRef.current = null;
      }
    };

    utterance.onend = resetBlink;
    utterance.onpause = resetBlink;
    utterance.oncancel = resetBlink;
    utterance.onerror = resetBlink;
    window.speechSynthesis.speak(utterance);
  }

  // Cleanup
  const stopAll = () => {
    if (loading) setLoading('');
    if (window.speechSynthesis) window.speechSynthesis.cancel();
    if (currentIconRef.current) currentIconRef.current = null;
    if (blinkIntervalRef.current) clearInterval(blinkIntervalRef.current);
    if (controllerRef.current) {
      controllerRef.current.abort();
      controllerRef.current = null;
    }
  }

  return {
    loading,
    messages,
    chatAI,
    session,
    lang,
    upFile,
    voiceChat,
    voiceStatus,
    thinkContent,
    newChat,
    removeChat,
    handleScroll,
    sendMessage,
    startVoice,
    stopVoice,
    copyChat,
    editChatStart,
    editChatStop,
    regenerateChat,
    speakChat,
    changeLang,
    setSessions,
    setMessages,
    setUpFile,
    upLoad
  };
}
