import { memo, useEffect, useRef, useState } from "react";
import { Button } from 'react-bootstrap';
import { marked } from 'marked';
import Prism from 'prismjs';
import ChatLoading from './chat-loading';
import ChatTime from "./chat-time";

function ChatMessage({loading, messages, regenerateChat, copyChat, editChatStart, editChatStop, speakChat, handleScroll, urlModule}) {

  const chatContainerRef = useRef(null);
  const [scrollOffset, setScrollOffset] = useState(0);
  const [isLoadingOldMessages, setIsLoadingOldMessages] = useState(false);

  const processMessageContent = (content, div) =>{
    if (marked) {
      marked.setOptions({
        highlight: function (code, lang) {
          if (window.Prism && Prism.languages[lang]) {
            return Prism.highlight(code, Prism.languages[lang], lang);
          }
          return code;
        },
        breaks: true,
        gfm: true,
        tables: true
      });

      const parsedContent = marked.parse(content);
      const container = document.getElementById(div);
      if (container) {
        Prism.highlightAllUnder(container);
      }

      return parsedContent;
    } else {
      content = LargestContentfulPaint
        .replace(/```(.*?)\n([\s\S]*?)```/g, function (match, language, code) {
          return '<pre><code class="language-' + language + '">' + code + '</code></pre>';
        })
        .replace(/`([^`]+)`/g, '<code>$1</code>')
        .replace(/^### (.*$)/gm, '<h3>$1</h3>')
        .replace(/^## (.*$)/gm, '<h2>$1</h2>')
        .replace(/^# (.*$)/gm, '<h1>$1</h1>')
        .replace(/\*\*(.*?)\*\*/g, '<strong>$1</strong>')
        .replace(/\*(.*?)\*/g, '<em>$1</em>')
        .replace(/^\s*\d+\.\s+(.*$)/gm, '<ol><li>$1</li></ol>')
        .replace(/^\s*[-*]\s+(.*$)/gm, '<ul><li>$1</li></ul>')
        .replace(/\[([^\]]+)\]\(([^)]+)\)/g, '<a href="$2" target="_blank">$1</a>')
        .replace(/\n/g, '<br>');

      return content;
    }
  }

  const handleScrollBody = () => {
    const container = chatContainerRef.current;
    if (!container) return;
    if (container.scrollTop < 50 && !isLoadingOldMessages) {
      const scrollOffset = container.scrollHeight - container.scrollTop;
      handleScroll();
      setScrollOffset(scrollOffset);
      setIsLoadingOldMessages(true);
    } else {
      isLoadingOldMessages && setIsLoadingOldMessages(false);
    }
  };

  useEffect(() => {
    const container = chatContainerRef.current;
    if (!container) return;
    if (!isLoadingOldMessages) {
      container.scrollTop = container.scrollHeight;
    } else {
      container.scrollTop = container.scrollHeight - scrollOffset;
    }
  }, [scrollOffset, messages])

  return (
    <div id='chat-body' className='chat-body flex-fill overflow-x-hidden overflow-y-auto p-2' ref={chatContainerRef} onScroll={handleScrollBody}>
      {messages.map((msg, idx) => (
        <div key={idx} className={`message ${msg.role === 'user' ? 'message-user' : 'message-bot'} mb-2`}>
          <div className={`message-body ${msg.role === 'user' ? 'flex-row' : 'flex-row-reverse'} d-flex justify-content-end`}>
            <div className={`${msg.role === 'user' ? 'pe-2' : 'ps-2'} position-relative d-flex justìy-content-end flex-column w-100 overflow-hidden`}>
              {msg.file?.length > 0 && msg.file.map((file, idx) => (
                <div key={idx} className='d-flex me-2 mb-1'>
                  <span className='d-inline-block ms-auto p-2 border rounded-3 bg-success bg-opacity-50'>{file.name}</span>
                </div>
              ))}
              <div className={`message-body-content position-relative d-inline-block pb-3 ${msg.role === 'user' ? 'ms-auto' : 'me-auto'}`}>
                {msg.content && <div className={`message-content position-relative z-1 ${msg.role === 'user' && 'bg-primary-subtle'} p-2 border rounded-2 shadow box-sizing-border-box`} dangerouslySetInnerHTML={{ __html: processMessageContent(msg.content, 'chat-body') }} onBlur={(e) => editChatStop(e, msg.datetime)} />}
                <div className={`message-action position-absolute z-3 bottom-0 align-items-center p-1 pe-0 rounded-2 ${msg.role === 'user' ? 'end-0' : 'start-0'}`}>
                  <ChatTime time={msg.datetime} />
                  <>
                    <Button variant='transparent' size='sm' className='d-flex align-items-center justify-content-center border-0 rounded-0 btn-scale' title={('Copy')} onClick={(e) => copyChat(e)}>
                      <i className='bi bi-clipboard text-danger lh-1'></i>
                    </Button>
                    {msg.role === 'user' ? (
                      <Button variant='transparent' size='sm' className='d-flex align-items-center justify-content-center border-0 rounded-0 btn-scale' title={('Regenerate')} onClick={(e) => regenerateChat(e)}>
                        <i className='bi bi-arrow-repeat text-danger lh-1'></i>
                      </Button>
                    ) : (
                      <>
                        <Button variant='transparent' size='sm' className='d-flex align-items-center justify-content-center border-0 rounded-0 btn-scale' title={('Speech')} onClick={(e) => speakChat(e)}>
                          <i className='btn-speak bi bi-volume-down text-danger fs-5 lh-1'></i>
                        </Button>
                        <Button variant='transparent' size='sm' className='d-flex align-items-center justify-content-center border-0 rounded-0 btn-scale' title={('Edit')} onClick={(e) => editChatStart(e)}>
                          <i className={`bi bi-pencil-square text-danger h-1`}></i>
                        </Button>
                      </>
                    )}
                  </>
                </div>
              </div>
              <div className="position-relative z-1 d-flex flex-column message-content-link">
                {msg.links?.map((htmlString, index) => (
                  <div key={index} className="d-flex mt-1 me-auto w-100" dangerouslySetInnerHTML={{ __html: htmlString }} />
                ))}
              </div>
            </div>
            <div className='message-auth d-flex flex-shrink-0 align-items justify-content-center overflow-hidden rounded-2 box-sizing-border-box'>
              {msg.role === 'user' ?
                <img src={window.drupalSettings?.deepseek?.avatar || `${urlModule}/icons/person-circle.svg`} alt="Person" width={40} height={40} />
                :
                <img src={window.drupalSettings?.deepseek?.logo_site || `${urlModule}/icons/deepseek.svg`} alt="ChatBot" width={40} height={40} />
              }
            </div>
          </div>
        </div>
      ))}
      <ChatLoading urlModule={urlModule} loading={loading}/>
    </div>
    );
}

export default memo(ChatMessage);
