(function ($, Drupal, drupalSettings, once) {
  'use strict';

  /**
   * Initializes the chat functionality.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.chatbox = {
    attach: function (context, settings) {
      once('chat-box-init', '.chatbox', context).forEach(function (chatContainer) {
        // Initialize core variables
        const chatAI = getChatAI();
        let currentSession = getCurrentSession(chatAI);
        const lang = drupalSettings.chat_box.voice || navigator.language || navigator.languages[0];
        const urlAgent = drupalSettings.chat_box.url_agent;
        const urlModule = drupalSettings.chat_box.url_module;
        const urlUpload = drupalSettings.chat_box.url_upload;
        let isLoading = false;
        let chatInitialized = false;
        let recognition = null;
        let isListening = false;

        // Cache DOM elements
        const $chatContent = $('#chat-content');
        const $messageInput = $('#message-input');
        const $sendButton = $('#send-button');
        const $chatFrame = $('#chat-frame');

        // Initialize event listeners
        initEventListeners();
        initButtonHandlers();

        // ===== Event Initialization =====
        function initEventListeners() {
          // Send button click
          $sendButton.click(handleSendMessage);

          // Enter key press (without Shift)
          $messageInput.keypress(function (e) {
            if (e.which === 13 && !e.shiftKey) {
              e.preventDefault();
              handleSendMessage();
            }
          });

          // Remove all messages
          $('#remove-messages').click(function () {
            localStorage.removeItem('chatAI');
            $chatContent.empty();
          });

          // New chat
          $('#new-chat').click(function () {
            if (!$chatContent.is(':empty')) {
              currentSession = newSession(chatAI);
              $chatContent.empty();
            }
          });

          // Toggle chat frame
          $('#chat-bot').click(function () {
            $chatFrame.toggle();
            if (!chatInitialized) {
              displayChats(chatAI, currentSession);
              chatInitialized = true;
              // Add scroll listener
              $chatContent.on('scroll', handleScroll);
            }
          });

          // Close chat frame
          $('#close-chat').click(function () {
            $chatFrame.hide();
          });

          // File attachment (placeholder)
          $('#attach-file').click(function (e) {
            e.preventDefault();
            alert('File attachment functionality to be implemented.');
          });

          // Voice recognition
          $('#voice').click(toggleVoiceRecognition);
        }

        // ===== Button Handlers =====
        function initButtonHandlers() {
          // Use event delegation for dynamic buttons
          document.addEventListener('click', function(e) {
            // Copy to clipboard
            const clipboardBtn = e.target.closest('.btn-clipboard');
            if (clipboardBtn) {
              e.preventDefault();
              clipboard(clipboardBtn);
              return;
            }

            // Text-to-speech
            const speakBtn = e.target.closest('.btn-speak');
            if (speakBtn) {
              e.preventDefault();
              textToSpeech(speakBtn);
              return;
            }

            // Regenerate bot reply
            const generateBtn = e.target.closest('.btn-regenerate');
            if (generateBtn) {
              e.preventDefault();
              regenerateReply(generateBtn);
              return;
            }
          });
        }

        // ===== Core Functions =====
        function handleSendMessage() {
          const message = $messageInput.val().trim();
          if (message !== '') {
            send(message, chatAI, currentSession);
          }
        }

        function handleScroll() {
          if ($chatContent.scrollTop() < 50 && !isLoading) {
            isLoading = true;
            const preSession = getPreviousSession(currentSession, chatAI);
            if (preSession) {
              displayChats(chatAI, preSession, true);
              currentSession = preSession;
            }
            setTimeout(() => isLoading = false, 200);
          }
        }

        function toggleVoiceRecognition(e) {
          e.preventDefault();
          const SpeechRecognition = window.SpeechRecognition || window.webkitSpeechRecognition;
          if (!SpeechRecognition) {
            console.error("Browser doesn't support Speech Recognition");
            return;
          }

          if (!recognition) {
            recognition = new SpeechRecognition();
            recognition.lang = lang;

            recognition.onstart = () => {
              isListening = true;
              $(this).text('🔴').addClass('recording');
            };

            recognition.onspeechend = () => {
              isListening = false;
              $(this).text('🎙').removeClass('recording');
              recognition.stop();
            };

            recognition.onend = () => {
              isListening = false;
              $(this).text('🎙').removeClass('recording');
            };

            recognition.onresult = (event) => {
              const transcript = event.results[0][0].transcript;
              if (transcript) {
                let text = $messageInput.val().trim() + ' ' + transcript.trim();
                $messageInput.val(text.trim());
              }
            };
          }
          if (isListening) {
            recognition.stop();
          } else {
            recognition.start();
          }
        }

        function send(message, chatAI, sessionId) {
          // Create and display user message
          const newChat = createChatMessage(sessionId, message, 'user');
          chatAI.chats.push(newChat);

          // Update session title if needed
          if (chatAI.sessions[sessionId].date === chatAI.sessions[sessionId].title) {
            chatAI.sessions[sessionId].title = trimText(message);
          }

          setChatAI(chatAI);
          displayMessage(newChat, sessionId);
          $messageInput.val('');

          // Get conversation history for this session
          const conversations = chatAI.chats
            .filter(chat => chat.session === sessionId)
            .map(chat => ({content: chat.content, role: chat.role}));

          // Create temporary bot message for streaming
          const botMessagePlaceholder = {
            session: sessionId,
            datetime: new Date().toISOString(),
            content: '',
            role: 'assistant'
          };

          displayMessage(botMessagePlaceholder);

          // Get the bot message container for streaming updates
          const $lastBotReply = $chatContent.find('.bot-reply').last();
          const botMessageDiv = document.createElement('div');
          botMessageDiv.className = 'bot-message markdown-content';
          $lastBotReply.append(botMessageDiv);

          let fullResponse = '';

          // Call API with streaming response
          fetchBotResponse(conversations, botMessageDiv, fullResponse)
            .then((finalResponse) => {
              // Save final response
              const botChat = createChatMessage(sessionId, finalResponse, 'assistant');
              chatAI.chats.push(botChat);
              setChatAI(chatAI);
            })
            .catch(error => {
              console.error('Error getting bot response:', error);
              botMessageDiv.innerHTML = `<div class="error">Error: ${error.message}</div>`;
            });
        }

        async function fetchBotResponse(conversations, botMessageDiv, fullResponse) {
          try {
            const response = await fetch(urlAgent, {
              method: 'POST',
              headers: {
                'Content-Type': 'application/json',
                'Accept': 'text/event-stream'
              },
              body: JSON.stringify(conversations)
            });

            if (!response.ok) {
              throw new Error('Network response was not ok');
            }

            // Create thinking div for displaying <think> content
            const thinkDiv = document.createElement('div');
            thinkDiv.className = 'think';
            botMessageDiv.appendChild(thinkDiv);

            const reader = response.body.getReader();
            const decoder = new TextDecoder();
            let inThink = false;

            // Process stream
            while (true) {
              const { done, value } = await reader.read();
              if (done) break;

              const chunk = decoder.decode(value, { stream: true });
              const lines = chunk.split('\n\n');

              for (const line of lines) {
                if (line.startsWith('data: ')) {
                  try {
                    const data = JSON.parse(line.substring(6));

                    // Handle special tags
                    if (data.content === "<think>") {
                      inThink = true;
                      continue;
                    }
                    if (data.content === "</think>") {
                      inThink = false;
                      continue;
                    }
                    if (data.content === "[DONE]") {
                      continue;
                    }
                    // Update display based on content type
                    if (inThink) {
                      thinkDiv.innerHTML += data.content;
                    } else if(typeof data.content === 'string') {
                      fullResponse += data.content;
                      botMessageDiv.innerHTML = processMessageContent(fullResponse);
                      highlightCode(botMessageDiv);
                    }

                    // Auto-scroll
                    const chatContent = botMessageDiv.closest('.chat-content');
                    if (chatContent) {
                      chatContent.scrollTop = chatContent.scrollHeight;
                    }
                  } catch (e) {
                    console.error('Error parsing SSE data:', e);
                  }
                }
              }
            }

            return fullResponse;
          } catch (error) {
            throw error;
          }
        }

        // ===== Helper Functions =====
        function createChatMessage(sessionId, content, role) {
          return {
            session: sessionId,
            datetime: new Date().toISOString(),
            content: content,
            role: role
          };
        }

        function getChatAI() {
          const chatAI = localStorage.getItem('chatAI');
          return chatAI ? JSON.parse(chatAI) : { sessions: {}, chats: [] };
        }

        function setChatAI(chatAI) {
          localStorage.setItem('chatAI', JSON.stringify(chatAI));
        }

        function getCurrentSession(chatAI) {
          const today = getFormattedDate();
          if (Object.keys(chatAI.sessions).length) {
            let lastEntry = Object.entries(chatAI.sessions).at(-1);
            let lastSession = lastEntry?.[1];
            if (lastSession && lastSession.date === today) {
              return lastEntry?.[0];
            }
          }
          return newSession(chatAI);
        }

        function newSession(chatAI) {
          const today = getFormattedDate();
          let session = Date.now().toString(36) + Math.random().toString(36).substr(2, 5);
          chatAI.sessions[session] = {
            date: today,
            title: today
          };
          localStorage.setItem('chatAI', JSON.stringify(chatAI));
          return session;
        }

        function getFormattedDate(offset = 0) {
          const date = new Date();
          date.setDate(date.getDate() - offset);
          return date.toISOString().split('T')[0];
        }

        function formatDate(dateString) {
          const today = new Date();
          today.setHours(0, 0, 0, 0);

          const inputDate = new Date(dateString);
          if (isNaN(inputDate.getTime())) {
            return 'Invalid date';
          }

          inputDate.setHours(0, 0, 0, 0);
          const oneDay = 24 * 60 * 60 * 1000;
          const diffDays = Math.round((today - inputDate) / oneDay);

          if (diffDays === 0) {
            return Drupal.t('Today');
          } else if (diffDays === 1) {
            return Drupal.t('Yesterday');
          } else {
            const day = String(inputDate.getDate()).padStart(2, '0');
            const month = String(inputDate.getMonth() + 1).padStart(2, '0');
            const year = inputDate.getFullYear();
            return `${day}/${month}/${year}`;
          }
        }

        function trimText(text, maxLength = 40) {
          if (text.length <= maxLength) return text;
          let trimmed = text.slice(0, maxLength);
          const lastSpaceIndex = trimmed.lastIndexOf(" ");
          if (lastSpaceIndex > 0) {
            trimmed = trimmed.slice(0, lastSpaceIndex);
          }
          return trimmed + "…";
        }

        function getPreviousSession(currentSessionId, chatAI) {
          const allSessions = [...new Set(chatAI.chats.map(chat => chat.session))];
          const currentIndex = allSessions.indexOf(currentSessionId);
          if (currentIndex > 0) {
            return allSessions[currentIndex - 1];
          }
          return null;
        }

        function displayChats(chatAI, currentSession, prepend = false) {
          const $chatContent = $('#chat-content');

          if (prepend) {
            $chatContent.prepend(`<div class="loading">${Drupal.t('Loading')}…</div>`);
          } else {
            $chatContent.empty();
          }

          displayDivider(chatAI.sessions[currentSession].date);

          // Get pre-height when it prepends
          const previousHeight = prepend ? $chatContent[0].scrollHeight : 0;
          const previousScrollTop = prepend ? $chatContent.scrollTop() : 0;

          // Display messages
          const sessionChats = chatAI.chats.filter(chat => chat.session === currentSession);
          sessionChats.forEach(chat => displayMessage(chat, currentSession, prepend));

          $chatContent.find('.loading').remove();

          // Adjust scroll position
          if (prepend) {
            const newHeight = $chatContent[0].scrollHeight;
            $chatContent.scrollTop(previousScrollTop + (newHeight - previousHeight));
          } else {
            $chatContent.scrollTop($chatContent[0].scrollHeight);
          }
        }

        function displayDivider(date = null) {
          if (date) {
            date = formatDate(date);
          }
          const $chatContent = $('#chat-content');
          let messageHtml = `<div class="divider text-center mx-3 mb-0">${date}</div>`;
          $chatContent.prepend(messageHtml);
        }

        function displayMessage(message, currentSession = '', prepend = false) {
          const $chatContent = $('#chat-content');
          const isUser = message.role === 'user';
          const alignment = isUser ? 'justify-content-end' : 'justify-content-start';
          const bgClass = isUser ? 'bg-primary text-white' : 'bg-body-tertiary';
          const avatarUrl = isUser ?
            `${urlModule}/icons/person-circle.svg` :
            `${urlModule}/icons/deepseek.svg`;
          const time = new Date(message.datetime).toLocaleTimeString([], {
            hour: '2-digit',
            minute: '2-digit'
          });

          const messageHtml = `
          <div class="d-flex flex-row ${alignment}">
            ${!isUser ? `<img src="${avatarUrl}" alt="avatar" style="width: 45px; height: 100%;">` : ''}
            <div class="chat-content">
              <div class="small p-2 ${isUser ? 'me-3 user-ask' : 'ms-3 bot-reply'} mb-1 rounded-3 ${bgClass}">
                ${processMessageContent(message.content)}
              </div>
              <div class="mb-3 rounded-3 text-muted ${isUser ? 'd-flex justify-content-end me-3' : 'ms-3'}">
                <time class="small">${time}</time>
                <a class="icon-link icon-link-hover ${isUser ? 'btn-regenerate ms-2' : 'btn-clipboard'}" href="#"
                   ${currentSession !== '' ? 'data-session="' + currentSession + '"' : ''}>
                  <i class="bi ${isUser ? 'bi-arrow-repeat' : 'bi-clipboard'}"></i>
                </a>
                ${!isUser ? '<a class="icon-link icon-link-hover btn-speak" href="#"><i class="bi bi-volume-up"></i></a>' : ''}
              </div>
            </div>
            ${isUser ? `<img src="${avatarUrl}" alt="avatar" style="width: 45px; height: 100%;">` : ''}
          </div>
        `;

          if (prepend) {
            $chatContent.prepend(messageHtml);
          } else {
            $chatContent.append(messageHtml);
          }

          $chatContent.scrollTop($chatContent[0].scrollHeight);
        }

        // ===== Utility Functions =====
        function processMessageContent(content) {
          if (window.marked) {
            marked.setOptions({
              highlight: function (code, lang) {
                if (window.Prism && Prism.languages[lang]) {
                  return Prism.highlight(code, Prism.languages[lang], lang);
                }
                return code;
              },
              breaks: true,
              gfm: true,
              tables: true
            });

            return marked.parse(content);
          } else {
            // Fallback markdown processing
            content = content
              // Code blocks
              .replace(/```(.*?)\n([\s\S]*?)```/g, function (match, language, code) {
                return '<pre><code class="language-' + language + '">' + code + '</code></pre>';
              })
              // Inline code
              .replace(/`([^`]+)`/g, '<code>$1</code>')
              // Headers
              .replace(/^### (.*$)/gm, '<h3>$1</h3>')
              .replace(/^## (.*$)/gm, '<h2>$1</h2>')
              .replace(/^# (.*$)/gm, '<h1>$1</h1>')
              // Bold and italic
              .replace(/\*\*(.*?)\*\*/g, '<strong>$1</strong>')
              .replace(/\*(.*?)\*/g, '<em>$1</em>')
              // Lists
              .replace(/^\s*\d+\.\s+(.*$)/gm, '<ol><li>$1</li></ol>')
              .replace(/^\s*[\-\*]\s+(.*$)/gm, '<ul><li>$1</li></ul>')
              // Links
              .replace(/\[([^\]]+)\]\(([^)]+)\)/g, '<a href="$2" target="_blank">$1</a>')
              // Line breaks
              .replace(/\n/g, '<br>');

            return content;
          }
        }

        function highlightCode(container) {
          if (window.Prism) {
            Prism.highlightAllUnder(container);
          }
        }

        function clipboard(clipboardBtn) {
          const chatContent = clipboardBtn.closest('.chat-content');
          const messageDiv = chatContent?.querySelector('div');
          if (!messageDiv) return;

          const textToCopy = messageDiv.innerText.trim();
          navigator.clipboard.writeText(textToCopy)
            .then(() => {
              clipboardBtn.innerHTML = '<i class="bi bi-clipboard-check"></i>';
              setTimeout(() => {
                clipboardBtn.innerHTML = '<i class="bi bi-clipboard"></i>';
              }, 2000);
            })
            .catch(err => {
              console.error('Failed to copy: ', err);
            });
        }

        function textToSpeech(speakBtn) {
          const chatContent = speakBtn.closest('.chat-content');
          const messageDiv = chatContent?.querySelector('div');
          if (!messageDiv) return;

          const textToSpeak = messageDiv.innerText.trim();
          let blinkInterval = speakBtn.getAttribute('data-blink-interval');

          // Cancel speaking if already speaking
          if (speechSynthesis.speaking) {
            speechSynthesis.cancel();
            if (blinkInterval) {
              clearInterval(parseInt(blinkInterval));
              speakBtn.removeAttribute('data-blink-interval');
              speakBtn.innerHTML = '<i class="bi bi-volume-up"></i>';
            }
            return;
          }

          const utterance = new SpeechSynthesisUtterance(textToSpeak);
          utterance.lang = lang;

          utterance.onstart = () => {
            // Create blinking effect
            blinkInterval = setInterval(() => {
              const icon = speakBtn.querySelector('i');
              icon.classList.toggle('bi-volume-up');
              icon.classList.toggle('bi-volume-down');
            }, 500);
            speakBtn.setAttribute('data-blink-interval', blinkInterval);
          };

          utterance.onend = () => {
            if (blinkInterval) {
              clearInterval(parseInt(speakBtn.getAttribute('data-blink-interval')));
              speakBtn.removeAttribute('data-blink-interval');
            }
            speakBtn.innerHTML = '<i class="bi bi-volume-up"></i>';
          };

          // Handle when speech is cancelled
          utterance.onpause = utterance.onend;
          utterance.oncancel = utterance.onend;

          speechSynthesis.speak(utterance);
        }

        function regenerateReply(generateBtn) {
          const chatContent = generateBtn.closest('.chat-content');
          const messageDiv = chatContent?.querySelector('div');
          if (!messageDiv) return;

          const message = messageDiv.innerText.trim();
          let chatAI = getChatAI();
          send(message, chatAI, currentSession);
        }
      });
    }
  };
})(jQuery, Drupal, drupalSettings, once);
