/**
 * @file
 * Provides deepseek AI chat functionality.
 */

(function (Drupal, drupalSettings, once, DOMPurify) {
  'use strict';

  /**
   * Initializes the chat functionality.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.deepseekChat = {
    attach: function (context, settings) {
      once('deepseek-chat-init', '#chat-container', context).forEach(function (chatContainer) {
        const sendBtn = document.getElementById('send-btn');
        const historyBtn = document.getElementById('history-btn');
        const newBtn = document.getElementById('new-btn');
        let langSelect = document.createElement('select');
        // Initialize Quill editor if not already initialized
        if (!window.quill) {
          // Launch Quill.js Editor
          window.quill = new Quill('#editor', {
            theme: 'snow',
            placeholder: Drupal.t('Ask anything...'),
            modules: {
              toolbar: {
                container: [
                  ['voice', 'image'],
                  ['bold', 'italic', 'underline', 'strike'],
                  ['blockquote', 'code-block'],
                  [{ 'list': 'ordered' }, { 'list': 'bullet' }],
                  ['link'],
                  ['clean'],
                ],
                handlers: {
                  'voice': function () {
                    document.querySelector('.ql-voice').click();
                  }
                }
              }
            }
          });
          document.querySelector('.ql-voice').innerHTML = '🎙️';
          voiceBtn();
          document.querySelector('.ql-toolbar').appendChild(langSelect);
          createLangSelect(langSelect);
          setLanguageSelect(langSelect);
          // Config Marked.js for Markdown rendering
          marked.setOptions({
            highlight: function (code, lang) {
              if (lang && hljs.getLanguage(lang)) {
                return hljs.highlight(code, {language: lang}).value;
              }
              return hljs.highlightAuto(code).value;
            },
            breaks: true
          });
        }

        // Process Send Button Click
        sendBtn.addEventListener('click', function () {
          const messageText = window.quill.root.innerHTML.trim().replace(/^"|"$/g, '');
          if (messageText) {
            sendMessage(messageText);
            window.quill.root.innerHTML = '';
          }
        });

        // Handle "Enter" key for sending messages
        window.quill.root.addEventListener('keydown', function (e) {
          if (e.key === 'Enter' && !e.shiftKey) {
            e.preventDefault();
            sendBtn.click();
          }
        });

        // Load chat history button handler
        historyBtn.addEventListener('click', function () {
          loadChatHistory();
        });

        // New chat session button handler
        newBtn.addEventListener('click', function () {
          localStorage.removeItem(getFormattedDate()); // Clear today's history
          chatContainer.innerHTML = '';
          addMessageToChat('🆕 ' + Drupal.t('Start a new session. You can ask questions!'), 'assistant');
        });

        // Show welcome message if the chat is empty
        if (chatContainer.innerHTML.trim() === '') {
          addMessageToChat(Drupal.t('Hello! I am AI assistant. You can ask me anything.'), 'assistant');
        }
      });
    }
  };

  // Speech Recognition.
  function voiceBtn() {
    const voiceButton = document.querySelector('.ql-voice');
    const SpeechRecognition = window.SpeechRecognition || window.webkitSpeechRecognition;
    if (!voiceButton || !SpeechRecognition) {
      console.error("Browser doesn't support Speech Recognition");
      return;
    }
    let recognition = null;
    let isListening = false;
    voiceButton.onclick = function () {
      if (isListening) {
        recognition.stop();
        return;
      }
      const quill = window.quill;
      const range = quill.getSelection();
      const index = range && range.index !== null ? range.index : quill.getLength();

      recognition = new SpeechRecognition();
      recognition.lang = localStorage.getItem('speechLanguage') || drupalSettings.deepseek.voice;
      if(recognition.lang === '') {
        recognition.lang = navigator.language || navigator.languages[0];
      }

      recognition.onstart = () => {
        isListening = true;
        voiceButton.innerHTML = '🔴';
        voiceButton.classList.add('recording');
      };

      recognition.onspeechend = () => {
        isListening = false;
        voiceButton.innerHTML = '🎙️';
        voiceButton.classList.remove('recording');
        recognition.stop();
      };
      recognition.onend = () => {
        isListening = false;
        voiceButton.innerHTML = '🎙️';
        voiceButton.classList.remove('recording');
      };

      recognition.onresult = (event) => {
        const transcript = event.results[0][0].transcript;
        if (transcript) {
          quill.insertText(index, transcript + ' ');
        }
      };

      recognition.start();

    };
    return voiceButton;
  }

  function setLanguageSelect(langList) {
    // Get language from localStorage or browser
    const browserLang = navigator.language || navigator.languages[0];
    langList.value = localStorage.getItem('speechLanguage') || browserLang;
    langList.addEventListener('change', () => {
      localStorage.setItem('speechLanguage', langList.value);
    });
  }
  function createLangSelect(langSelect) {
    const languages = [
      { code: 'en-US', name: 'English' },
      { code: 'fr-FR', name: 'Français' },
      { code: 'it-IT', name: 'Italiano' },
      { code: 'es-ES', name: 'Español' },
      { code: 'ca-ES', name: 'Català' },
      { code: 'gl-ES', name: 'Galego' },
      { code: 'pt-PT', name: 'Português' },
      { code: 'pt-BR', name: 'Brasil' },
      { code: 'vi-VN', name: 'Tiếng Việt' },
      { code: 'af-ZA', name: 'Afrikaans' },
      { code: 'bs-BA', name: 'Bosanski' },
      { code: 'id-ID', name: 'Bahasa' },
      { code: 'jv-ID', name: 'Basa Java' },
      { code: 'cy-GB', name: 'Cymraeg' },
      { code: 'da-DK', name: 'Dansk' },
      { code: 'de-DE', name: 'Deutsch' },
      { code: 'et-EE', name: 'Eesti' },
      { code: 'eu-ES', name: 'Euskera' },
      { code: 'fa-IR', name: 'Farsi' },
      { code: 'fil-PH', name: 'Filipino' },
      { code: 'ga-IE', name: 'Gaeilge' },
      { code: 'hr-HR', name: 'Hrvatski' },
      { code: 'sw-KE', name: 'Kiswahili' },
      { code: 'kk-KZ', name: 'Қазақ' },
      { code: 'lt-LT', name: 'Lietuvių' },
      { code: 'lv-LV', name: 'Latviešu' },
      { code: 'mt-MT', name: 'Malti' },
      { code: 'hu-HU', name: 'Magyar' },
      { code: 'nl-NL', name: 'Nederlands' },
      { code: 'uz-UZ', name: 'Oʻzbekcha' },
      { code: 'pl-PL', name: 'Polski' },
      { code: 'sq-AL', name: 'Shqip' },
      { code: 'sv-SE', name: 'Svenska' },
      { code: 'fi-FI', name: 'Suomi' },
      { code: 'cs-CZ', name: 'Čeština' },
      { code: 'is-IS', name: 'Íslenska' },
      { code: 'ro-RO', name: 'Română' },
      { code: 'tr-TR', name: 'Türkçe' },
      { code: 'sk-SK', name: 'Slovenčina' },
      { code: 'el-GR', name: 'Ελληνικά' },
      { code: 'az-AZ', name: 'Azərbaycanca' },
      { code: 'bg-BG', name: 'Български' },
      { code: 'sr-RS', name: 'Српски' },
      { code: 'mk-MK', name: 'Македонски' },
      { code: 'mn-MN', name: 'монгол' },
      { code: 'ru-RU', name: 'Русский' },
      { code: 'uk-UA', name: 'Українська' },
      { code: 'ja-JP', name: '日本語' },
      { code: 'ko-KR', name: '한국어' },
      { code: 'th-TH', name: 'ไทย' },
      { code: 'km-KH', name: 'ភាសាខ្មែរ' },
      { code: 'lo-LA', name: 'ພາສາລາວ' },
      { code: 'ar-SA', name: 'العربية' },
      { code: 'ms-MY', name: 'بهاس ملايو' },
      { code: 'ps-AF', name: 'پښتو' },
      { code: 'am-ET', name: 'አማርኛ' },
      { code: 'bn-IN', name: 'বাংলা' },
      { code: 'he-IL', name: 'עברית' },
      { code: 'hy-AM', name: 'Հայերեն' },
      { code: 'ka-GE', name: 'ქართული ენა' },
      { code: 'my-MM', name: 'ဗမာစကား' },
      { code: 'si-LK', name: 'සිංහල' },
      { code: 'gu-IN', name: 'ગુજરાતી' },
      { code: 'hi-IN', name: 'हिन्दी' },
      { code: 'kn-IN', name: 'ಕನ್ನಡ' },
      { code: 'ml-IN', name: 'മലയാളം' },
      { code: 'mr-IN', name: 'मराठी' },
      { code: 'or-IN', name: 'ଓଡିଆ' },
      { code: 'pa-IN', name: 'ਪੰਜਾਬੀ' },
      { code: 'ta-IN', name: 'தமிழ்' },
      { code: 'te-IN', name: 'తెలుగు' },
      { code: 'ur-IN', name: 'اردو' },
      { code: 'ne-NP', name: 'नेपाली' },
      { code: 'zh-CN', name: '中文' },
      { code: 'zh-HK', name: '中文 (香港)' },
      { code: 'zh-TW', name: '繁體中文' },
    ];
    langSelect.id = 'langSelect';
    langSelect.title = Drupal.t('Set language to speech');
    languages.forEach(lang => {
      const option = document.createElement('option');
      option.value = lang.code;
      option.textContent = lang.name;
      langSelect.appendChild(option);
    });
    return langSelect;
  }
  /**
   * Sends a message to the AI and handles the response.
   *
   * @param {string} message - The message to send.
   * @param {string} time - The time sent
   */
  function sendMessage(message, time = '') {
    const chatContainer = document.getElementById('chat-container');

    // Add user message to chat
    addMessageToChat(message, 'user', time);
    let sessionId = saveMessageToHistory('user', message);
    // Show "thinking" animation
    const thinkingDiv = document.createElement('div');
    thinkingDiv.className = 'message thinking-message';
    thinkingDiv.innerHTML = `<div class="thinking"></div>`;
    chatContainer.appendChild(thinkingDiv);
    chatContainer.scrollTop = chatContainer.scrollHeight;

    try {
      // Extract plain text from HTML
      const tempDiv = document.createElement('div');
      tempDiv.innerHTML = message;
      let conversation = getSession(sessionId);

      // Call URL controller AI connect.
      let url = drupalSettings.deepseek.url;

      // Create bot message to show stream
      let botMessageDiv = null, messageDiv =null ;
      let fullResponse = '';

      // Send request.
      fetch(url, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'Accept': 'text/event-stream'
        },
        body: JSON.stringify(conversation)
      }).then(response => {
        // Check response
        if (!response.ok) {
          throw new Error('Network response was not ok');
        }

        // Remove thinking animation
        if (chatContainer.contains(thinkingDiv)) {
          chatContainer.removeChild(thinkingDiv);
        }

        // Create new div to show reply from controller
        botMessageDiv = document.createElement('div');
        const timeSpan = document.createElement('time');
        timeSpan.className = 'd-inline-block me-2';
        const copy = document.createElement('a');
        copy.className = 'icon-link icon-link-hover btn-clipboard';
        copy.href = '#';
        const speak = document.createElement('a');
        speak.className = 'icon-link icon-link-hover btn-speaker ms-2';
        speak.href = '#';
        messageDiv = document.createElement('div');
        messageDiv.className = 'message assistant-message small p-2 rounded-3 my-2 shadow-sm bg-body-tertiary w-75';
        botMessageDiv.className = 'message-content';
        botMessageDiv.innerHTML = '';
        let thinkDiv = document.createElement('div');
        thinkDiv.className = 'think';
        botMessageDiv.appendChild(thinkDiv);
        messageDiv.appendChild(botMessageDiv);
        messageDiv.appendChild(timeSpan);
        messageDiv.appendChild(copy);
        messageDiv.appendChild(speak);
        chatContainer.appendChild(messageDiv);

        // Process stream from response
        const reader = response.body.getReader();
        const decoder = new TextDecoder();
        let inThink = false;

        return new Promise((resolve, reject) => {
          function push() {
            reader.read().then(({ done, value }) => {
              if (done) {
                resolve(fullResponse);
                return;
              }

              // Decode chunk
              const chunk = decoder.decode(value, { stream: true });
              const lines = chunk.split('\n\n');
              for (const line of lines) {
                if (line.startsWith('data: ')) {
                  try {
                    const data = line.substring(6);

                    let parsedData;
                    try {
                      parsedData = JSON.parse(data);
                    } catch (e) {
                      console.log('Not valid JSON:', data);
                      continue;
                    }
                    if (parsedData.content === '[DONE]') {
                      chatContainer.scrollTop = chatContainer.scrollHeight;
                      timeSpan.innerHTML = new Date().toLocaleTimeString([], {hour: '2-digit', minute: '2-digit'});
                      copy.innerHTML = '<i class="bi bi-clipboard"></i>';
                      speak.innerHTML = '<i class="bi bi-volume-up"></i>';
                      initButtonHandlers();
                      break;
                    }
                    // Handle <think> tags
                    if (parsedData.content === "<think>") {
                      inThink = true;
                      continue;
                    }
                    if (parsedData.content === "</think>") {
                      inThink = false;
                      continue;
                    }

                    // Update chat box
                    if (inThink) {
                      thinkDiv.innerHTML += parsedData.content;
                    } else {
                      fullResponse += parsedData.content;
                      botMessageDiv.innerHTML = processMessageContent(fullResponse);
                      highlightCode(botMessageDiv); // Highlight code after each update
                    }

                    // Scroll to container
                    chatContainer.scrollTop = chatContainer.scrollHeight;
                  } catch (e) {
                    console.error('Error parsing SSE data:', e);
                  }
                }
              }

              push();
            }).catch(reject);
          }

          push();
        });
      })
      .then((finalResponse) => {
        // Store to history
        saveMessageToHistory('assistant', finalResponse);
      })
      .catch(error => {
        if (chatContainer.contains(thinkingDiv)) {
          chatContainer.removeChild(thinkingDiv);
        }
        addMessageToChat('⚠ ' + Drupal.t('Error connecting to AI. Please try again later!'), 'assistant');
      });
    } catch (error) {
      if (chatContainer.contains(thinkingDiv)) {
        chatContainer.removeChild(thinkingDiv);
      }
      addMessageToChat('⚠ ' + Drupal.t('An unexpected error occurred. Please try again.'), 'assistant');
    }
  }

  function highlightCode(container) {
    if (window.Prism) {
      // Highlight all code blocks within the specified container
      Prism.highlightAllUnder(container);
    }
  }

// Process message with support Markdown
  function processMessageContent(content) {
    if (window.marked) {
      // Config marked to support code highlighting
      marked.setOptions({
        highlight: function (code, lang) {
          if (window.Prism && Prism.languages[lang]) {
            return Prism.highlight(code, Prism.languages[lang], lang);
          }
          return code;
        },
        breaks: true, // break to new line
        gfm: true,    // Support GitHub Flavored Markdown
        tables: true  // Support GFM
      });

      // Parse Markdown to HTML
      return marked.parse(content);
    } else {
      // Process code blocks
      content = content.replace(/```(.*?)\n([\s\S]*?)```/g, function (match, language, code) {
        return '<pre><code class="language-' + language + '">' + code + '</code></pre>';
      });

      // Process inline code.
      content = content.replace(/`([^`]+)`/g, '<code>$1</code>');

      // Process header
      content = content.replace(/^### (.*$)/gm, '<h3>$1</h3>');
      content = content.replace(/^## (.*$)/gm, '<h2>$1</h2>');
      content = content.replace(/^# (.*$)/gm, '<h1>$1</h1>');

      // Process bold and italic.
      content = content.replace(/\*\*(.*?)\*\*/g, '<strong>$1</strong>');
      content = content.replace(/\*(.*?)\*/g, '<em>$1</em>');

      // Process list.
      content = content.replace(/^\s*\d+\.\s+(.*$)/gm, '<ol><li>$1</li></ol>');
      content = content.replace(/^\s*[\-\*]\s+(.*$)/gm, '<ul><li>$1</li></ul>');

      // Process link.
      content = content.replace(/\[([^\]]+)\]\(([^)]+)\)/g, '<a href="$2" target="_blank">$1</a>');

      // Process break line.
      content = content.replace(/\n/g, '<br>');

      return content;
    }
  }

  /**
   * Adds a message to the chat container.
   *
   * @param {string} message - The message content.
   * @param {string} sender - The sender type ('user' or 'assistant').
   * @param time - Time sent.
   */
  function addMessageToChat(message, sender, time= '') {
    const chatContainer = document.getElementById('chat-container');
    const messageDiv = document.createElement('div');
    messageDiv.className = `message ${sender}-message `;

    const messageContent = document.createElement('div');
    messageContent.className = 'message-content ';

    if (sender === 'assistant') {
      // Process Markdown for AI replies
      messageContent.innerHTML = DOMPurify.sanitize(marked.parse(message));

      // Apply syntax highlighting to code blocks
      Prism.highlightAllUnder(messageContent);
      messageDiv.className += 'small p-2 rounded-3 my-2 shadow-sm bg-body-tertiary w-75';
    } else {
      messageDiv.className += 'd-flex flex-row-reverse';
      messageContent.className += 'small p-2 my-1 rounded-3 shadow-sm bg-primary text-white';
      // For user messages, just use the HTML directly
      messageContent.innerHTML = DOMPurify.sanitize(message);
    }

    messageDiv.appendChild(messageContent);
    if (time !== '' && sender === 'assistant') {
      const timeSpan = document.createElement('time');
      timeSpan.className = 'd-inline-block me-2';
      timeSpan.innerHTML = new Date(time).toLocaleTimeString([], {hour: '2-digit', minute: '2-digit'});
      messageDiv.appendChild(timeSpan);
      const copy = document.createElement('a');
      copy.className = 'icon-link icon-link-hover btn-clipboard';
      copy.href = '#';
      copy.innerHTML = '<i class="bi bi-clipboard"></i>';
      messageDiv.appendChild(copy);
      const speak = document.createElement('a');
      speak.className = 'icon-link icon-link-hover btn-speaker ms-2';
      speak.href = '#';
      speak.innerHTML = '<i class="bi bi-volume-up"></i>';
      messageDiv.appendChild(speak);
    }
    chatContainer.appendChild(messageDiv);
    chatContainer.scrollTop = chatContainer.scrollHeight;
  }

  /**
   * Copy to clipboard and text to speech.
   */
  function initButtonHandlers() {
    document.addEventListener('click', function(e) {
      if (e.target.closest('.btn-clipboard')) {
        e.preventDefault();
        const button = e.target.closest('.btn-clipboard');
        const messageDiv = button.parentElement;
        const messageContent = messageDiv.querySelector(':first-child');
        const textToCopy = messageContent.innerText.trim();
        navigator.clipboard.writeText(textToCopy)
          .then(() => {
            button.innerHTML = '<i class="bi bi-clipboard-check"></i>';
            setTimeout(() => {
              button.innerHTML = '<i class="bi bi-clipboard"></i>';
            }, 2000);
          })
          .catch(err => {
            console.error('Failed to copy: ', err);
          });
      }

      if (e.target.closest('.btn-speaker')) {
        e.preventDefault();
        const speakBtn = e.target.closest('.btn-speaker');
        const messageDiv = speakBtn.parentElement;
        if (!messageDiv) return;
        const messageContent = messageDiv.querySelector(':first-child');
        const textToSpeak = messageContent.innerText.trim();
        let blinkInterval = speakBtn.getAttribute('data-blink-interval');
        // Cancel speaking if already speaking
        if (speechSynthesis.speaking) {
          speechSynthesis.cancel();
          if (blinkInterval) {
            clearInterval(parseInt(blinkInterval));
            speakBtn.removeAttribute('data-blink-interval');
            speakBtn.innerHTML = '<i class="bi bi-volume-up"></i>';
          }
          return;
        }
        const utterance = new SpeechSynthesisUtterance(textToSpeak);

        const browserLang = navigator.language || navigator.languages[0];
        utterance.lang = localStorage.getItem('speechLanguage') || browserLang;

        utterance.onstart = () => {
          // Create blinking effect
          blinkInterval = setInterval(() => {
            const icon = speakBtn.querySelector('i');
            icon.classList.toggle('bi-volume-up');
            icon.classList.toggle('bi-volume-down');
          }, 500);
          speakBtn.setAttribute('data-blink-interval', blinkInterval);
        };

        utterance.onend = () => {
          if (blinkInterval) {
            clearInterval(parseInt(speakBtn.getAttribute('data-blink-interval')));
            speakBtn.removeAttribute('data-blink-interval');
          }
          speakBtn.innerHTML = '<i class="bi bi-volume-up"></i>';
        };

        // Handle when speech is cancelled
        utterance.onpause = utterance.onend;
        utterance.oncancel = utterance.onend;
        speechSynthesis.speak(utterance);
      }
    });
  }

  /**
   * Gets a formatted date string.
   *
   * @param {number} offset - Days to offset from today (default: 0).
   * @return {string} Formatted date string (YYYY-MM-DD).
   */
  function getFormattedDate(offset = 0) {
    const date = new Date();
    date.setDate(date.getDate() - offset);
    return date.toISOString().split('T')[0];
  }

  /**
   * Random session key.
   * @returns {string}
   */
  function generateSessionId() {
    return Date.now().toString(36) + Math.random().toString(36).substr(2, 5);
  }

  /**
   * Saves a message to the local storage history.
   *
   * @param {string} role - The role of the message sender ('user' or 'bot').
   * @param {string} message - The message content.
   */
  function saveMessageToHistory(role, message) {
    const chatAI = JSON.parse(localStorage.getItem('chatAI')) || {sessions: {}, chats: []};
    const today = getFormattedDate();

    // Find session today
    let currentEntry = Object.entries(chatAI.sessions).find(([id, session]) => session.date === today);
    let currentId = currentEntry?.[0];
    let currentSession = currentEntry?.[1];
    let id = currentId ?? generateSessionId();
    if (!currentSession) {
      chatAI.sessions[id] = {
        date: today,
        title: today
      };
    }

    // Add message to history
    chatAI.chats.push({
      session: id,
      datetime: new Date().toISOString(),
      content: stripTags(message),
      role: role
    });

    if (chatAI.sessions[id].date === chatAI.sessions[id].title) {
      chatAI.sessions[id].title = trimText(stripTags(message));
    }

    localStorage.setItem('chatAI', JSON.stringify(chatAI));
    return id;
  }
  // Remove html.
  function stripTags(input) {
    let doc = new DOMParser().parseFromString(input, "text/html");
    return doc.body.textContent || "";
  }

  /**
   * Get conversation by id.
   */
  function getSession(sessionId) {
    const chatAI = JSON.parse(localStorage.getItem('chatAI')) || {sessions: {}, chats: []};
    return chatAI.chats
      .filter(chat => chat.session === sessionId)
      .map(chat => ({ content: chat.content, role: chat.role }));
  }

  /**
   * Cut string max 30 chars
   */
  function trimText(text, maxLength = 30) {
    if (text.length <= maxLength) return text;
    let trimmed = text.slice(0, maxLength);
    const lastSpaceIndex = trimmed.lastIndexOf(" ");
    if (lastSpaceIndex > 0) {
      trimmed = trimmed.slice(0, lastSpaceIndex);
    }
    return trimmed + "…";
  }

  /**
   * Loads and displays chat history in the history modal.
   */
  function loadChatHistory() {
    const chatAI = JSON.parse(localStorage.getItem('chatAI')) || {sessions: {}, chats: []};
    const today = getFormattedDate();
    const yesterday = getFormattedDate(1);

    // History in 7 days
    const last7Days = Array.from({length: 6}, (_, i) => getFormattedDate(i + 2));

    // Group session by date
    const groupedSessions = {
      [today]: {title: 'Today', sessions: []},
      [yesterday]: {title: 'Yesterday', sessions: []},
      'last7Days': {title: 'Last 7 Days', sessions: []}
    };

    // Classified session
    Object.entries(chatAI.sessions).forEach(([id, session]) => {
      session.id = id;
      if (session.date === today) {
        groupedSessions[today].sessions.push(session);
      } else if (session.date === yesterday) {
        groupedSessions[yesterday].sessions.push(session);
      } else if (last7Days.includes(session.date)) {
        groupedSessions.last7Days.sessions.push(session);
      }
    });

    // Create HTML
    let historyHtml = '';

    Object.values(groupedSessions).forEach(group => {
      if (group.sessions.length === 0) return;

      historyHtml += `<div class="history-group">
                     <h4>${Drupal.t(group.title)}</h4>`;
      historyHtml +='<ul class="list-unstyled">';
      group.sessions.forEach(session => {
        historyHtml += `<li class="session d-flex justify-content-between" id="${session.id}">
            <a class="history-load-session text-decoration-none" href="#" data-id="${session.id}">${session.title}</a>
            <div class="dropdown">
                <button class="btn btn-sm btn-light dropdown-toggle" type="button" data-bs-toggle="dropdown" aria-expanded="false"><i class="bi bi-three-dots-vertical"></i></button>
                <ul class="dropdown-menu" >
                  <li><a class="dropdown-item history-rename" href="#" data-id="${session.id}"><i class="bi bi-pencil-square"></i> ${Drupal.t('Rename')}</a></li>
                  <li><a class="dropdown-item history-delete text-danger" href="#" data-id="${session.id}"><i class="bi bi-trash"></i> ${Drupal.t('Delete')}</a></li>
                </ul>
            </div>
        </li>`;
      });
      historyHtml +='</ul>';

      historyHtml += `</div>`;
    });

    const historyContainer = document.getElementById("history-modal-content");
    historyContainer.innerHTML = historyHtml || `<p>${Drupal.t('No chat history available.')}</p>`;

    // Delete history
    const historyBtnDelete = document.getElementsByClassName('history-delete');
    Array.from(historyBtnDelete).forEach(button => {
      button.addEventListener('click', function(event) {
        const sessionId = this.dataset.id;
        if (sessionId) {
          const sessionsData = JSON.parse(localStorage.getItem('chatAI')) || {};
          sessionsData.chats = sessionsData.chats.filter(chat => chat.session !== sessionId);
          delete sessionsData.sessions[sessionId];
          localStorage.setItem('chatAI', JSON.stringify(sessionsData));
          document.getElementById(sessionId).remove();
        }
      });
    });

    // Rename history
    const historyBtnRename = document.getElementsByClassName('history-rename');
    Array.from(historyBtnRename).forEach(button => {
      button.addEventListener('click', function(event) {
        const sessionId = this.dataset.id;
        if (sessionId) {
          const titleElement = document.querySelector(`.history-title[data-id="${sessionId}"]`);
          titleElement.contentEditable = true;
          titleElement.focus();
          // Cancel rename.
          titleElement.addEventListener('blur', function() {
            rename(titleElement);
          });
          // Change title by Enter
          titleElement.addEventListener('keydown', function(event) {
            if (event.key === 'Enter') {
              event.preventDefault();
              rename(titleElement);
            }
          });
          function rename(titleElement) {
            const newTitle = titleElement.textContent.trim();
            titleElement.contentEditable = false;
            // Replace title
            const sessionsData = JSON.parse(localStorage.getItem('chatAI')) || {};
            if (sessionsData.sessions && sessionsData.sessions[sessionId]) {
              sessionsData.sessions[sessionId].title = newTitle;
              localStorage.setItem('chatAI', JSON.stringify(sessionsData));
            }
          }
        }
      });
    });

    // Load history session
    const historyLoadSession = document.getElementsByClassName('history-load-session');
    Array.from(historyLoadSession).forEach(button => {
      button.addEventListener('click', function(event) {
        const sessionId = this.dataset.id;
        if (sessionId) {
          chatAI.chats
          .filter(chat => chat.session === sessionId)
          .forEach(chat => {
            addMessageToChat(chat.content, chat.role, chat.datetime);
          });
          initButtonHandlers();
        }
      });
    });

  }

})(Drupal, drupalSettings, once, DOMPurify);
