import { Plugin } from 'ckeditor5/src/core';
import { IconCheck, IconCancel, IconRobotPencil } from '@ckeditor/ckeditor5-icons';
import { createDropdown, addListToDropdown, ContextualBalloon, View, ButtonView} from 'ckeditor5/src/ui';
import { Collection } from 'ckeditor5/src/utils';
import templateIcon from '../../../../icons/deepseek.svg';

export default class InsertPrompt extends Plugin {
  async init() {
    const editor = this.editor;

    // Allowed class and data-* on div
    editor.config.set('htmlSupport', {
      allow: [
        {
          name: 'div',
          attributes: {
            class: true,
            'data-prompt-id': true,
            'data-library': true,
          },
          classes: true,
        },
      ],
    });

    // Get prompts name from editor config.
    const prompts = this.editor.config.get('prompts');
    editor.ui.componentFactory.add('InsertPrompt', (locale) => {
      const dropdown = createDropdown(locale);
      dropdown.buttonView.set({
        label: editor.t('Insert prompt'),
        withText: false,
        icon: IconRobotPencil,
        tooltip: true
      });

      // Add prompt name to dropdown
      addListToDropdown(dropdown, this._createPromptItems(prompts));

      dropdown.on('execute', async (event) => {
        await this._onExecute(event);
      });

      return dropdown;
    });
    // Create balloon panel to add class
    this._balloon = editor.plugins.get(ContextualBalloon);
  }

  _createPromptItems(prompts) {
    const collection = new Collection();

    const templateElement = {
      label: Drupal.t('Ask me'),
      withText: true,
      icon: IconRobotPencil,
      tooltip: Drupal.t('Deepseek AI'),
      promptId: 'ask'
    };
    collection.add({
      type: 'button',
      model: templateElement
    });

    Object.entries(prompts).forEach(([id, prompt]) => {
      let promptId = prompt.id, label = prompt.title;
      const templateElement = {
        label: label,
        withText: true,
        tooltip: label,
        promptId: promptId
      };
      collection.add({
        type: 'button',
        model: templateElement
      });
    });

    return collection;
  }
  async _onExecute(event) {
    const promptId = event.source.promptId;

    if (promptId === 'ask') {
      // With "ask", open balloon to get input from user
      this._showBalloonForAsk();
    }
    else {
      // For other options, proceed immediately with the corresponding prompt.
      this._processPrompt(promptId);
    }
  }

  /**
   * Show balloon to get input "ask"
   */
  _showBalloonForAsk() {
    const editor = this.editor;
    const contextualBalloon = editor.plugins.get('ContextualBalloon');

    // Create content balloon
    const balloonView = this._createBalloonContent();
    // Add balloon to ContextualBalloon
    contextualBalloon.add({
      view: balloonView,
      position: this._getBalloonPosition(),
    });

    // Show balloon
    contextualBalloon.showStack('main');
    document.getElementById('ask-me').focus();
  }

  /**
   * Process prompt and send request to backend
   * @param {string} promptId - ID prompt ("ask" or prompt)
   * @param {string|null} userInput - Input if "ask"
   */
  async _processPrompt(promptId, userInput = null) {
    const editor = this.editor;
    const selection = editor.model.document.selection;
    const isSelection = !selection.isCollapsed;
    let streamPosition;

    // Prepare data to send.
    const prompts = editor.config.get('prompts') || {};
    const promptText = promptId === 'ask' ? userInput : (prompts[promptId]?.prompt || '');

    // Get text to send before inserting "Loading..."
    //const textToSend = isSelection ? this._getSelectedText() : this._getEntireText();
    const textToSend = isSelection ? this._getSelectedText() : '';
    const fullText = [];
    const systemText = prompts[promptId]?.system?.trim();
    // Optional: Add system message first if it exists.
    if (systemText) {
      fullText.push({
        role: 'system',
        content: systemText
      });
    }
    // Add user message.
    fullText.push({
      role: 'user',
      content: promptText + "\n" + textToSend
    });

    // Request to backend
    const urlGetContent = editor.config.get('ai_connect');
    try {
      const response = await fetch(urlGetContent, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'Accept': 'text/event-stream'
        },
        body: JSON.stringify(fullText)
      });

      if (!response.ok) {
        throw new Error('Network response was not ok');
      }

      // Process stream
      const reader = response.body.getReader();
      const decoder = new TextDecoder();
      let htmlContent = '';
      let inThink = false;
      // Position to stream
      editor.model.change(writer => {
        if (isSelection && promptId === 'ask') {
          // With "ask" and selected text: replace text
          const range = selection.getFirstRange();
          writer.remove(range);
          streamPosition = range.start;
        } else if (!isSelection) {
          // Without selected text: insert in cursor
          streamPosition = selection.getFirstPosition();
        } else {
          // with another option: replace selected text
          const range = selection.getFirstRange();
          writer.remove(range);
          streamPosition = range.start;
        }
      });

      // Get stream reply
      while (true) {
        const { done, value } = await reader.read();
        if (done) break;
        const chunk = decoder.decode(value, { stream: true });
        const lines = chunk.split('\n\n');
        for (const line of lines) {
          if (line.startsWith('data: ')) {
            try {
              const data = JSON.parse(line.substring(6));
              if (data.content === '<think>') {
                inThink = true;
              } else if (data.content === '</think>') {
                inThink = false;
              } else if (data.content === '[DONE]') {
                // End of stream
              } else if (!inThink) {
                const previousContent = htmlContent;
                htmlContent += data.content;
                if (htmlContent !== previousContent) {
                  editor.model.change(writer => {
                    const root = editor.model.document.getRoot();
                    const range = writer.createRangeIn(root);
                    writer.remove(range);
                    const viewFragment = editor.data.processor.toView(this._processMessageContent(htmlContent));
                    const modelFragment = editor.data.toModel(viewFragment);
                    writer.insert(modelFragment, writer.createPositionAt(root, 0));
                  });
                }
              }
            } catch (e) {
              console.error('Error parsing SSE data:', e);
            }
          }
        }
      }
    } catch (error) {
      console.error('Error processing prompt:', error);
      editor.model.change(writer => {
        if (isSelection) {
          const range = writer.createRangeOn(loadingElement);
          writer.remove(range);
        } else {
          editor.setData(Drupal.t('Error occurred while processing your request.'));
        }
      });
    }
  }

  /**
   * Get selected text
   * @returns {string} Selected text
   */
  _getSelectedText() {
    const selection = this.editor.model.document.selection;
    if (selection.isCollapsed) return '';
    const range = selection.getFirstRange();
    let text = '';
    for (const item of range.getItems()) {
      if (item.is('text') || item.is('textProxy')) {
        text += item.data;
      }
    }
    return text;
  }

  /**
   * Get document in ckeditor
   * @returns {string} All text of ckeditor
   */
  _getEntireText() {
    const html = this.editor.getData();
    const doc = new DOMParser().parseFromString(html, 'text/html');
    return doc.body.textContent || '';
  }
  _createBalloonContent(position) {
    const editor = this.editor;
    const balloonView = new View();

    // Create textarea to ask AI
    const textarea = document.createElement('textarea');
    textarea.placeholder = Drupal.t('Ask me') + ' ...';
    textarea.style.width = '500px';
    textarea.id = 'ask-me';

    // Btn voice with icon voice
    const voiceButton = new ButtonView();
    voiceButton.set({
      label: '🎙️',
      ariaLabel: Drupal.t('Voice'),
      tooltip: Drupal.t('Voice'),
      withText: true,
      type: 'button',
      class: 'ck-button-voice',
    });
    voiceButton.render();

    // Btn Send with icon check
    const sendButton = new ButtonView();
    sendButton.set({
      label: Drupal.t('Send'),
      icon: IconCheck,
      tooltip: true,
      class: 'ck-button-save'
    });
    sendButton.render();

    // Button Cancel with icon cancel
    const cancelButton = new ButtonView();
    cancelButton.set({
      label: Drupal.t('Cancel'),
      icon: IconCancel,
      tooltip: true,
      class: 'ck-button-cancel'
    });
    cancelButton.render();

    // Create container for textarea and button
    const contentContainer = document.createElement('div');
    const br = document.createElement('br');
    contentContainer.style.padding = '10px';
    contentContainer.appendChild(textarea);
    contentContainer.appendChild(br);
    contentContainer.appendChild(voiceButton.element);
    contentContainer.appendChild(sendButton.element);
    contentContainer.appendChild(cancelButton.element);

    // Set element for balloonView
    balloonView.element = contentContainer;

    textarea.addEventListener('load', () => {
      textarea.focus();
    });

    let recognition = null;
    let isListening = false;
    voiceButton.on('execute', () => {
      if (isListening) {
        recognition.stop();
        return;
      }
      const SpeechRecognition = window.SpeechRecognition || window.webkitSpeechRecognition;
      recognition = new SpeechRecognition();
      recognition.lang = editor.config.get('voice');
      if (recognition.lang === undefined || recognition.lang === '') {
        recognition.lang = navigator.language || navigator.languages[0];
      }

      recognition.onstart = () => {
        isListening = true;
        voiceButton.element.innerHTML = '🔴';
      };

      recognition.onspeechend = () => {
        isListening = false;
        voiceButton.element.innerHTML = '🎙️';
        recognition.stop();
      };
      recognition.onend = () => {
        isListening = false;
        voiceButton.element.innerHTML = '🎙️';
      };

      recognition.onresult = (event) => {
        const transcript = event.results[0][0].transcript;
        if (transcript) {
          textarea.value = textarea.value + ' ' + transcript;
          textarea.value = textarea.value.trim();
        }
      };
      try {
        recognition.start();
      } catch (error) {
        console.error("Voice error:", error);
      }
    });
    sendButton.on('execute', () => {
      const userInput = textarea.value;
      if (userInput.trim()) {
        this._processPrompt('ask', userInput);
        // Close balloon after send
        editor.plugins.get('ContextualBalloon').remove(balloonView);
      }
    });
    cancelButton.on('execute', () => {
      // Close balloon click cancel
      editor.plugins.get('ContextualBalloon').remove(balloonView);
    });

    return balloonView;
  }

  // Process message with support Markdown
  _processMessageContent(content) {
    if (window.marked) {
      // Config marked to support code highlighting
      marked.setOptions({
        highlight: function (code, lang) {
          if (window.Prism && Prism.languages[lang]) {
            return Prism.highlight(code, Prism.languages[lang], lang);
          }
          return code;
        },
        breaks: true, // break to new line
        gfm: true,    // Support GitHub Flavored Markdown
        tables: true  // Support GFM
      });

      // Parse Markdown to HTML
      return marked.parse(content);
    } else {
      // Process code blocks
      content = content.replace(/```(.*?)\n([\s\S]*?)```/g, function (match, language, code) {
        return '<pre><code class="language-' + language + '">' + code + '</code></pre>';
      });

      // Process inline code.
      content = content.replace(/`([^`]+)`/g, '<code>$1</code>');

      // Process header
      content = content.replace(/^### (.*$)/gm, '<h3>$1</h3>');
      content = content.replace(/^## (.*$)/gm, '<h2>$1</h2>');
      content = content.replace(/^# (.*$)/gm, '<h1>$1</h1>');

      // Process bold and italic.
      content = content.replace(/\*\*(.*?)\*\*/g, '<strong>$1</strong>');
      content = content.replace(/\*(.*?)\*/g, '<em>$1</em>');

      // Process list.
      content = content.replace(/^\s*\d+\.\s+(.*$)/gm, '<ol><li>$1</li></ol>');
      content = content.replace(/^\s*[\-\*]\s+(.*$)/gm, '<ul><li>$1</li></ul>');

      // Process link.
      content = content.replace(/\[([^\]]+)\]\(([^)]+)\)/g, '<a href="$2" target="_blank">$1</a>');

      // Process break line.
      content = content.replace(/\n/g, '<br>');

      return content;
    }
  }

  /**
   * Get balloon position
   * @returns {Object} position for ContextualBalloon
   */
  _getBalloonPosition() {
    const editor = this.editor;
    const selection = editor.model.document.selection;
    const range = selection.getFirstRange();
    const viewRange = editor.editing.mapper.toViewRange(range);
    return {
      target: editor.editing.view.domConverter.viewRangeToDom(viewRange)
    };
  }

  static get requires() {
    return [ContextualBalloon];
  }

}
